// -----------------------------------------------------------------------------
// mut4sym.cc
// -----------------------------------------------------------------------------
#include <iostream>                                   // C++ I/O
#include <exception>
#include <fstream>                                    // C++ file I/O
#include <assert.h>

extern "C" {
#include <nemo.h>                                     // NEMO basics
  int io_nemo(char *, char *,...);
}

inline const void *  abortFailedAlloc(const void *);
int applySymOn(const int, const int, const int, int, const float);

using namespace std; // prevent writing statment like 'std::cerr'

float 
  * x    = NULL, * xo   = NULL, 
  * v    = NULL, * vo   = NULL,
  * m    = NULL, * mo   = NULL,
  * t    = NULL;
int
  * nbody= NULL, * no   = NULL;

//------------------------------------------------------------------------------
inline const void * abortFailedAlloc(const void * p)
{
  if (!p) {
    cerr << "Allocation failed, aborting program....\n";
    exit(1);
  }
  return p;
}
//------------------------------------------------------------------------------
int applySymOn(const int a,const int b,const int c, int offset, const float mscale)
{
  cerr << "Offset ["<<offset<<"]\n";
  for (int i=0;i<*nbody;i++) {
    // Velocities
    vo[(i+offset)*3+0] = v[i*3+0]*a;
    vo[(i+offset)*3+1] = v[i*3+1]*b;
    vo[(i+offset)*3+2] = v[i*3+2]*c;
    // Positions
    xo[(i+offset)*3+0] = x[i*3+0]*a;
    xo[(i+offset)*3+1] = x[i*3+1]*b;
    xo[(i+offset)*3+2] = x[i*3+2]*c;
    // Masses
    mo[(i+offset)    ] = m[i    ]*1./mscale;
  }
  offset+=*nbody;
  return offset;
}
//------------------------------------------------------------------------------
int buildAxialSym(char * in,char * out, const float mscale)
{
  int status;

  status=io_nemo(in,"read,float,info,n,t,m,x,v",&nbody,&t,&m,&x,&v);
  try {
    vo = new float[(*nbody) * 3 * 4];
    xo = new float[(*nbody) * 3 * 4];
    mo = new float[(*nbody)     * 4];
    no = new int                    ;
  }
  catch(std::bad_alloc& ) {
    cerr << "Allocation failed\n";
    exit(1);
  }
  *no=*nbody * 4;

  if (status) {
    int offset=0;
    offset=applySymOn( 1, 1, 1,offset,mscale);
    offset=applySymOn( 1, 1,-1,offset,mscale);
    offset=applySymOn(-1,-1, 1,offset,mscale);
    offset=applySymOn(-1,-1,-1,offset,mscale);

    assert(offset == *no);
    // Save particles
    status=io_nemo(out,"save,float,info,n,t,m,x,v",&no,&t,&mo,&xo,&vo);

    if (!status) {
      cerr << "Problem occur during SAVING io_nemo call...\n";
    } 
    else {
      io_nemo(out,"close");
    }
  }
  else {
    cerr << "Problem occur during READING io_nemo call...\n";
  }
  return 1;
}
//------------------------------------------------------------------------------
// NEMO parameters
::string defv[] = {  // use `::'string because of 'using namespace std'
  "in=???\n             input NEMO file                                    ",
  "out=???\n            output NEMO                                        ",
  "mscale=1\n           mass factor scaling                                ",
  "VERSION=1.0\n        compiled on "__DATE__"/"__TIME__                  "",
  NULL
};
::string usage="Build 4 symmetry axials around each particles";


//------------------------------------------------------------------------------

int main(int argc, char ** argv )
{
  ::string in,out;

  //   start  NEMO             
  initparam(argv,defv);                            
  
  // Get parameters
  in     = getparam("in");
  out    = getparam("out");
  const float mscale = getdparam("mscale");
  //
  buildAxialSym(in,out,mscale);

  //   finish NEMO             
  finiparam();                                     
}
// ----------- End Of [mult4sym.c] ------------------------------------
